{ Main view, where most of the application logic takes place.

  Feel free to use this code as a starting point for your own projects.
  This template code is in public domain, unlike most other CGE code which
  is covered by BSD or LGPL (see https://castle-engine.io/license). }
unit GameViewMain;

interface

uses Classes,
  CastleVectors, CastleComponentSerialize,
  CastleUIControls, CastleControls, CastleKeysMouse, CastleScene,
  CastleTransform;

type
  { Main view, where most of the application logic takes place. }
  TViewMain = class(TCastleView)
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    LabelFps: TCastleLabel;
    Player: TCastleBox;
    PlayerBody: TCastleRigidBody;
    Beam1Body: TCastleRigidBody;
    Beam2Body: TCastleRigidBody;
  private
    const
      ThrustForce = 50 * 100;
      PlayerSpeed = 100;
    var
      beam1Speed: Integer;
      beam2Speed: Integer;
      isBeamStarted: Boolean;
    function InputLeft: Boolean;
    function InputRight: Boolean;
    function InputUp: Boolean;
    function InputDown: Boolean;
    //procedure BeamCollisionEnter(const CollisionDetails: TPhysicsCollisionDetails);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
    function Press(const Event: TInputPressRelease): Boolean; override;
  end;

var
  ViewMain: TViewMain;

implementation

uses SysUtils;

function TViewMain.InputLeft: Boolean;
begin
  Result :=
    Container.Pressed.Items[keyA] or
    Container.Pressed.Items[keyArrowLeft];
end;

function TViewMain.InputRight: Boolean;
begin
  Result :=
    Container.Pressed.Items[keyD] or
    Container.Pressed.Items[keyArrowRight];
end;

function TViewMain.InputUp: Boolean;
begin
  Result :=
    Container.Pressed.Items[keyW] or
    Container.Pressed.Items[keyArrowUp];
end;

function TViewMain.InputDown: Boolean;
begin
  Result :=
    Container.Pressed.Items[keyS] or
    Container.Pressed.Items[keyArrowDown];
end;
{procedure TViewMain.BeamCollisionEnter(const CollisionDetails: TPhysicsCollisionDetails);
begin
  beamSpeed := beamSpeed * -1;
end;}

{ TViewMain ----------------------------------------------------------------- }

constructor TViewMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gameviewmain.castle-user-interface';
end;

procedure TViewMain.Start;
begin
  inherited;
  //Player.TranslationXY := Vector2(0, -457);
  beam1Speed := 600;
  beam2Speed := 500;
  isBeamStarted := False;
  //BeamBody.OnCollisionEnter := @BeamCollisionEnter;
end;

procedure TViewMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
var
  direction: TVector3;
begin
  inherited;
  { This virtual method is executed every frame (many times per second). }
  Assert(LabelFps <> nil, 'If you remove LabelFps from the design, remember to remove also the assignment "LabelFps.Caption := ..." from code');
  LabelFps.Caption := 'FPS: ' + Container.Fps.ToString;
  PlayerBody.LinearVelocity := TVector3.Zero;
  // * Beam1 - each frame an AddForce
  // * By some games a good idea, but not by all games
  // * This doesn't work well, why does the item go back too quickly?
  Beam1Body.AddForce(Vector3(beam1Speed * ThrustForce, 0, 0), True);
  //BeamBody.LinearVelocity := Vector3(beamSpeed * ThrustForce, 0, 0);
  // * Beam2 - only once an AddForce when isBeamStarted
  // * Normally, in Unity I must use an AddForce only once
  // * The Physics Material does the calculation and the speed remains the same
  // * You say: This is automatically in de rigidbody
  // * But I don't see what the object is supposed to do,
  // * if it's in there automatically
  if not isBeamStarted then
  begin
    Beam2Body.AddForce(Vector3(beam2Speed * ThrustForce, 0, 0), True);
    isBeamStarted := True;
	end;
	if InputUp then
  begin
    direction := Vector3(0, PlayerSpeed, 0);
    PlayerBody.LinearVelocity := direction; // * SecondsPassed;
    //PlayerBody.AddForce(direction * ThrustForce, True);
    //Player.TranslationXY := Player.TranslationXY + Vector2(0, 5);
  end;
  if InputDown then
  begin
    direction := Vector3(0, -PlayerSpeed, 0);
    //PlayerBody.AddForce(direction * ThrustForce, True);
    PlayerBody.LinearVelocity := direction; // * SecondsPassed;
    //Player.TranslationXY := Player.TranslationXY + Vector2(0, -5);
	end;
  if InputLeft then
  begin
    direction := Vector3(-PlayerSpeed, 0, 0);
    //PlayerBody.AddForce(direction * ThrustForce, True);
    PlayerBody.LinearVelocity := direction; // * SecondsPassed;
    //Player.TranslationXY := Player.TranslationXY + Vector2(-5, 0);
	end;
  if InputRight then
  begin
    direction := Vector3(PlayerSpeed, 0, 0);
    //PlayerBody.AddForce(direction * ThrustForce, True);
    PlayerBody.LinearVelocity := direction; // * SecondsPassed;
    //Player.TranslationXY := Player.TranslationXY + Vector2(5, 0);
	end;
end;

function TViewMain.Press(const Event: TInputPressRelease): Boolean;
begin
  Result := inherited;
  if Result then Exit; // allow the ancestor to handle keys

  { This virtual method is executed when user presses
    a key, a mouse button, or touches a touch-screen.

    Note that each UI control has also events like OnPress and OnClick.
    These events can be used to handle the "press", if it should do something
    specific when used in that UI control.
    The TViewMain.Press method should be used to handle keys
    not handled in children controls.
  }

  // Use this to handle keys:
  {
  if Event.IsKey(keyXxx) then
  begin
    // DoSomething;
    Exit(true); // key was handled
  end;
  }

end;

end.
